local SLASH_TEXTURE = Resources.load_texture("spell_dreamsword.png")
local BLADE_TEXTURE = Resources.load_texture("elem_sword_blades.png")
local AUDIO = Resources.load_audio("sfx.ogg")

function card_mutate(player, index)
	local card = player:field_card(index);

	local list = player:field_cards()
	local length = #list

	local i = index + 1
	local next_card;

	while i < length do
		next_card = player:field_card(i)
		if next_card and next_card.damage > 0 then
			card.damage = card.damage + math.ceil(next_card.damage * 0.5)
			card.boosted_damage = card.boosted_damage + math.ceil(next_card.damage * 0.5)
			player:remove_field_card(i)
		else
			i = i + 1
		end
	end
	player:set_field_card(index, card)
end

function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_SWING")
	action:set_lockout(ActionLockout.new_animation())
	action:set_card_properties(props)
	action.on_execute_func = function(self, user)
		self:add_anim_action(2, function()
			local hilt = self:create_attachment("HILT")
			local hilt_sprite = hilt:sprite()
			hilt_sprite:set_texture(actor:texture())
			hilt_sprite:set_layer(-2)
			hilt_sprite:use_root_shader(true)

			local hilt_anim = hilt:animation()
			hilt_anim:copy_from(actor:animation())
			hilt_anim:set_state("HILT")

			local blade = hilt:create_attachment("ENDPOINT")
			local blade_sprite = blade:sprite()
			blade_sprite:set_texture(BLADE_TEXTURE)
			blade_sprite:set_layer(-1)

			local blade_anim = blade:animation()
			blade_anim:load("elem_sword_blades.animation")
			blade_anim:set_state("WOOD")
		end)

		self:add_anim_action(3, function()
			local facing = user:facing()
			-- Starter tile is 1 in front of the user
			local tile = user:get_tile(facing, 1)
			-- List of all other tiles to hit & flash.
			local tile_list = {
				tile:get_tile(Direction.Up, 1), tile, tile:get_tile(Direction.join(Direction.Up, facing), 1),
				tile, tile:get_tile(facing, 1),
				tile:get_tile(Direction.Down, 1), tile:get_tile(Direction.join(Direction.Down, facing), 1),
			}

			local field = actor:field()

			local sword = create_slash(user, props, tile_list)
			field:spawn(sword, tile)

			local fx = Artifact.new()
			fx:set_facing(sword:facing())
			local anim = fx:animation()
			fx:set_texture(SLASH_TEXTURE)
			anim:load("spell_dreamsword.animation")
			anim:set_state("DEFAULT")
			anim:on_complete(function()
				fx:erase()
				sword:erase()
			end)
			actor:field():spawn(fx, tile)
		end)
	end
	return action
end

function create_slash(user, props, tile_list)
	local facing = user:facing()

	local spell = Spell.new(user:team())

	spell:set_facing(facing)

	spell:set_tile_highlight(Highlight.Solid)

	spell:set_hit_props(
		HitProps.new(
			props.damage,
			Hit.Impact | Hit.Flinch | Hit.Flash,
			props.element,
			user:context(),
			Drag.None
		)
	)

	spell.on_update_func = function(self)
		local current_tile = self:current_tile()
		for i = 1, #tile_list, 1 do
			if tile_list[i] and not tile_list[i]:is_edge() then
				tile_list[i]:set_highlight(Highlight.Flash)
				tile_list[i]:attack_entities(self)
			end
		end
	end

	spell.can_move_to_func = function(tile)
		return true
	end

	spell.on_collision_func = function(self, other)
		self:erase()
	end

	Resources.play_audio(AUDIO)

	return spell
end
